@@ -7,9 +7,13 @@
  * - 'tag_selected', param: tag_id
  * - 'tag_selected_and_note', param: tag_id, note_id
  * - 'notebooks_visibility', param: boolean
+ * - 'refresh_lasts_notes'
  * 
  * - 'sync_over' when a joplin synch has finished and notes list should be refreshed.
  * - "sync_started" when joplin sync starts and notes list and note view shoudl be cleared
+ * 
+ * - "please hide notes history"
+ * - "please show notes history"
  */
 class SideBar extends EventEmitter{
     constructor() {
@@ -18,8 +22,11 @@ class SideBar extends EventEmitter{
         this.last_tag_id = null;
         this.last_notebook_id = null;
         this.reselect_after_reload = 0;
+        this.reselect_after_reload_also_notes = false;
 
-        $("#notebook_toolbox .icon-plus").on("click", () => { console.log("cliccccc") });
+        this.notebook_addition(true);
+        $("#notebook_toolbox .icon-plus1").on("click", () => { this.create_notebook() });
+        $("#notebook_toolbox .icon-pencil").on("click", () => { this.edit_or_delete_notebook() });
     }
 
     
@@ -44,16 +51,27 @@ class SideBar extends EventEmitter{
     }
 
     /**
-     * @return null if no note selected, otherwise the id as string
+     * @return null if no notebook selected, otherwise the id as string
      */
     get_selected_notebook_id() {
-        let selected = $('#notebooks_tree_inner').tree('getSelectedNode', this.last_notebook_id);
+        let selected = $('#notebooks_tree_inner').tree('getSelectedNode');
         if (selected) {
             return selected.id;
         }
         return null;
     }
     
+    /**
+     * @return null if no notebook selected, otherwise the name as string
+     */
+    get_selected_notebook_name() {
+        let selected = $('#notebooks_tree_inner').tree('getSelectedNode');
+        if (selected) {
+            return selected.name;
+        }
+        return null;
+    }
+    
     /**
      *
      */
@@ -95,6 +113,7 @@ class SideBar extends EventEmitter{
 
         if (this.last_notes_source == "notebook") {
             const node = $('#notebooks_tree_inner').tree('getNodeById', this.last_notebook_id);
+            $('#notebooks_tree_inner').tree('selectNode', null);
             $('#notebooks_tree_inner').tree('selectNode', node);
         }
 
@@ -105,6 +124,23 @@ class SideBar extends EventEmitter{
         else {
             this.toggle_accordion("tags_ctn");
         }
+
+        // also reload notes ?
+        if (this.reselect_after_reload_also_notes == true) {
+            this.reselect_after_reload_also_notes = false;
+            super.emit("notebook_selected", this.last_notebook_id); // for now, only last_notebook_id is involved when reselect_after_reload_also_notes == true
+        }
+    }
+    
+    /**
+     * Select the given notebook id, and unfold notebook section.
+     * @param {*} notebook_id 
+     */
+    select_notebook(notebook_id) {
+        this.reselect_after_reload_also_notes = false;
+        this.last_notes_source = "notebook";
+        this.last_notebook_id = notebook_id;
+        this.reselect();
     }
 
     /**
@@ -137,6 +173,14 @@ class SideBar extends EventEmitter{
                 super.emit("notebook_selected", node_id);
             }
         });
+
+        tree.on('tree.select', (event) => {
+                if (event.node) {
+                    // node was selected
+                    this.notebook_addition(true);
+                }
+            }
+        );
         
         // prevent unselection of notebook:
         $('#notebooks_tree_inner').bind(
@@ -238,6 +282,7 @@ class SideBar extends EventEmitter{
         console.log("reload after sync");
         this.reset_sync_dirty();
         this.not_sync_header_back();
+        super.emit("please show notes history");
         if (this.sync_polling != null) {
             this.sync_polling.removeListener("sync_over", this.reload_side_bar_after_sync_handler);
         }
@@ -257,6 +302,7 @@ class SideBar extends EventEmitter{
             this.sync_polling.on("sync_over", this.reload_side_bar_after_sync_handler);
             this.sync_polling.pause_emit();
             this.not_sync_header_readonly();
+            super.emit("please hide notes history");
         }
         this.udpate_sync_started();
         super.emit("sync_started");
@@ -269,6 +315,7 @@ class SideBar extends EventEmitter{
         }
         ).fail(() => {
             this.not_sync_header_back();
+            super.emit("please show notes history");
             if (this.sync_polling != null) {
                     this.resume_emit();
               }  
@@ -399,14 +446,31 @@ class SideBar extends EventEmitter{
 
     /**
      * show or hide the + button to add a notebook.
+     * Also show / add trash icon according to notebook selected or not.
      * @param {boolean} visible 
      */
     notebook_addition(visible) {
+        let notebook_selected = this.get_selected_notebook_id();
+
         if (visible) {
-            $("#notebook_toolbox").show(400);
+            if (notebook_selected != null) {
+                $("#notebook_toolbox .icon-pencil").show(400);
+                $("#notebook_toolbox").show(400);
+            }
+            else {
+                $("#notebook_toolbox .icon-pencil").hide(0, () => { $("#notebook_toolbox").show(400); });
+            }
+            
         }
         else {
-            $("#notebook_toolbox").hide(400);
+            $("#notebook_toolbox").hide(400, () => {
+                if (notebook_selected != null) {
+                    $("#notebook_toolbox .icon-pencil").show(0);
+                }
+                else {
+                    $("#notebook_toolbox .icon-pencil").hide(0);
+                }
+            });
         }
     }
 
@@ -454,6 +518,187 @@ class SideBar extends EventEmitter{
     reset_sync_dirty () {
         $(".dirty").remove();
     }
+
+    /**
+     * 
+     */
+    create_notebook() {
+        let selected_notebook_id = this.get_selected_notebook_id();
+        let parent_id = null;
+        if (selected_notebook_id) {
+            parent_id = selected_notebook_id;
+        }
+
+        //
+        // Popup
+        //
+        if (parent_id == null) {
+            $("#new_nb_with_parent_radio").hide();
+            $('label[for="new_nb_with_parent_radio"]').hide();
+            $("#new_nb_root_radio").prop('checked', true);
+        }
+        else {
+            $("#new_nb_with_parent_radio").show();
+            $('label[for="new_nb_with_parent_radio"]').show();
+            $("#new_nb_with_parent_name").html(this.get_selected_notebook_name());
+            $("#new_nb_root_radio").prop('checked', false);
+            $("#new_nb_with_parent_radio").prop('checked', true);
+        }
+        $("#notebook_create_popup").modal({fadeDuration: 100 });
+
+        // unregister events after modal is closed.
+        $("#notebook_create_popup").on($.modal.CLOSE , function (event, modal) {
+            $("#notebook_create_popup").find("*").off();
+            $("#notebook_create_popup").off();
+        });
+        
+        $("#notebook_create_popup .button_Cancel").on("click", () => { 
+            $.modal.close();
+        });
+        
+        $("#notebook_create_popup .button_OK").on("click", () => { 
+            if ($("#new_nb_root_radio").prop('checked')) {
+                parent_id = "0";
+            }
+            let title = $("#nb_title").val();
+            if (title == "") {
+                $("#notebook_create_popup_error_no_title").modal({ fadeDuration: 100 });
+            }
+            else {
+                $.ajax({
+                    url: '/joplin/notebooks/' + parent_id + "/",
+                    type: 'post',
+                    headers: { "X-CSRFToken": csrftoken },
+                    data: JSON.stringify({ "parent_id": parent_id, "title": title }),
+                    complete: (new_id) => {
+                        this.set_sync_dirty();
+                        this.last_notes_source = "notebook";
+                        this.last_notebook_id = new_id.responseText;
+                        this.reselect_after_reload = 2;
+                        this.reselect_after_reload_also_notes = true;
+                        this.reload();
+                    }
+                });
+                $.modal.close();
+            }
+        });
+
+        $("#notebook_create_popup").on($.modal.OPEN, function (event, modal) {
+            $("#nb_title").val('');
+            $("#nb_title").focus();
+        });
+    }
+
+    edit_or_delete_notebook() {
+        // Display edit or delete popup
+        let name = this.get_selected_notebook_name();
+        $("#edit_or_delete_notebook_nb_name").html(name);
+        $("#edit_or_delete_notebook_popup").modal({ fadeDuration: 100 });
+
+        // unregister events after modal is closed.
+        $("#edit_or_delete_notebook_popup").on($.modal.CLOSE, function (event, modal) {
+            $("#edit_or_delete_notebook_popup").find("*").off();
+            $("#edit_or_delete_notebook_popup").off();
+        });
+
+        // on edit:
+        $("#edit_or_delete_notebook_popup .button_edit").on("click", () => {
+            $.modal.close();
+            this.rename_notebook();
+        });
+
+
+        // on delete:
+        $("#edit_or_delete_notebook_popup .button_delete").on("click", () => {
+            $.modal.close();
+            this.delete_notebook();
+        });
+    }
+
+    delete_notebook() {
+        //
+        // display confirm popup
+        //
+        let name = this.get_selected_notebook_name();
+        $("#notebook_delete_popup_nb_name").html(name);
+        $("#notebook_delete_popup").modal({ fadeDuration: 100 });
+
+        // cancel button close modal:
+        $("#notebook_delete_popup .button_Cancel").on("click", () => {
+            $.modal.close();
+        });
+
+
+        // unregister events after modal is closed.
+        $("#notebook_delete_popup").on($.modal.CLOSE, function (event, modal) {
+            $("#notebook_delete_popup").find("*").off();
+            $("#notebook_delete_popup").off();
+        });
+
+        // Delete button deletes:
+        $("#notebook_delete_popup .button_OK").on("click", () => {
+            let notebook_id = this.get_selected_notebook_id();
+            $.ajax({
+                url: '/joplin/notebooks/' + notebook_id + "/delete/",
+                type: 'post',
+                headers: { "X-CSRFToken": csrftoken },
+                complete: () => {
+                    this.set_sync_dirty();
+                    this.last_notes_source = "notebook";
+                    this.last_notebook_id = null;
+                    this.reselect_after_reload = 2;
+                    this.reselect_after_reload_also_notes = true;
+                    this.reload();
+                    super.emit("refresh_lasts_notes");
+                }
+            });
+            $.modal.close();
+        });
+    }
+
+    rename_notebook() {
+        //
+        // display rename notebook popup
+        //
+        let name = this.get_selected_notebook_name();
+        $("#notebook_rename_popup .notebook_rename_popup_nb_name").html(name);
+        $("#notebook_rename_popup input").val(name);
+        $("#notebook_rename_popup").modal({ fadeDuration: 100 });
+
+        // cancel button close modal:
+        $("#notebook_rename_popup .button_Cancel").on("click", () => {
+            $.modal.close();
+        });
+
+        // Set focus
+        $("#notebook_rename_popup").on($.modal.OPEN, function (event, modal) {
+            $('#notebook_rename_popup input').focus();
+        });
+
+
+        // unregister events after modal is closed.
+        $("#notebook_rename_popup").on($.modal.CLOSE, function (event, modal) {
+            $("#notebook_rename_popup").find("*").off();
+            $("#notebook_rename_popup").off();
+        });
+
+        // Rename button renames:
+        $("#notebook_rename_popup .button_OK").on("click", () => {
+            let notebook_id = this.get_selected_notebook_id();
+            $.ajax({
+                url: '/joplin/notebooks/' + notebook_id + "/rename/",
+                type: 'post',
+                data: JSON.stringify({ "title": $('#notebook_rename_popup input').val() }),
+                headers: { "X-CSRFToken": csrftoken },
+                complete: () => {
+                    this.set_sync_dirty();
+                    this.reselect_after_reload = 2;
+                    this.reload();
+                }
+            });
+            $.modal.close();
+        });
+    }
 }
 
 /**
